/* 
(C) OOMusou 2008 http://oomusou.cnblogs.com

Filename    : DE2_70.v
Compiler    : Quartus II 8.1
Description : Demo how to use TRDB-D5M with LTM on DE2-70 with Nios II
Release     : 12/18/2008 1.0
*/

`default_nettype none

module LTM_Controller (
  // Avalon clock interface siganals
  input                        csi_clockreset_clk,        // avalon-mm clk
  input                        csi_clockreset_reset_n,    // avalon-mm reset_n
  // Signals for Avalon-MM master port
  output [DATAWIDTH-1:0]       avm_m1_address,            // avalon-mm master address
  output [BYTEENABLEWIDTH-1:0] avm_m1_byteenable,         // avalon-mm master byteenable
  output                       avm_m1_read,               // avalon-mm master read
  input                        avm_m1_readdatavalid,      // avalon-mm master readdatavalid
  input  [DATAWIDTH-1:0]       avm_m1_readdata,           // avalon-mm master readdata
  input                        avm_m1_waitrequest,        // avalon-mm master waitrequest
  // Signals for Avalon-MM slave port
  input  [ADDRWIDTH-1:0]       avs_s1_address,            // avalon-mm slave address
  input  [BYTEENABLEWIDTH-1:0] avs_s1_byteenable,         // avalon-mm slave byteenable
  input                        avs_s1_write,              // avalon-mm slave write
  input  [DATAWIDTH-1:0]       avs_s1_writedata,          // avalon-mm slave writedata
  input                        avs_s1_read,               // avalon-mm slave read
  output [DATAWIDTH-1:0]       avs_s1_readdata,           // avalon-mm slave readdata
  // LTM couduit input
  input                        coe_ltm_export_iCLK_50,    // 50MHz
  input                        coe_ltm_export_iRST0,      // reset delay 0
  input                        coe_ltm_export_iRST2,      // reset delay 2
  // LTM couduit output 
  output                       coe_ltm_export_oLTM_CLK,   // ltm clk
  output [LTM_DATAWIDTH-1:0]   coe_ltm_export_oR,         // ltm R
  output [LTM_DATAWIDTH-1:0]   coe_ltm_export_oG,         // ltm G
  output [LTM_DATAWIDTH-1:0]   coe_ltm_export_oB,         // ltm B
  output                       coe_ltm_export_oHD,        // ltm h.sync
  output                       coe_ltm_export_oVD,        // ltm v.sync
  output                       coe_ltm_export_oDEN,       // ltm data enable
  output                       coe_ltm_export_oSCLK,      // ltm I2S clk
  inout                        coe_ltm_export_ioSDAT,     // ltm I2S data
  output                       coe_ltm_export_oSCEN,      // ltm I2S clk enable
  output                       coe_ltm_export_oFIFO_FULL, // for debug use only (ltm fifo empty)
  output                       coe_ltm_export_oFIFO_EMPTY // for debug use only (ltm fifo full)
);

// LTM parameter
parameter LTM_DATAWIDTH   = 8;                // ltm data width

// Avalon-MM parameter
parameter DATAWIDTH       = 32;               // width of databus
parameter BYTEENABLEWIDTH = 4;                // width of byteenable
parameter ADDRESSBASE     = 32'h0080_0000;    // SDRAM frame buffer start address
parameter LENGTH          = 800 * 480;        // LTM width : 800, height = 480
// Todo!! FIFODEPTH parameter
parameter FIFODEPTHLOG2   = 12;               // FIFO width of depth
// Todo!! FIFOFULLVALUE parameter
parameter ADDRWIDTH       = 3;                // avalon-mm slave address width

localparam FIFOALMOSTFULL = FIFODEPTH - FIFOFULLVALUE; // FIFO almost full value

// LTM wire
wire ltm_nclk; // ltm clk
wire read;     // sdram read request

// fifo wire
wire                       fifo_wrclk;  // fifo write clk
wire                       fifo_aclr;   // fifo asynchronous clk
wire                       fifo_wrreq;  // fifo write request
wire [DATAWIDTH-1:0]       fifo_wrdata; // fifo write data
wire                       fifo_rdclk;  // fifo read clk
wire                       fifo_rereq;  // fifo read request
wire [DATAWIDTH-1:0]       fifo_rddata; // fifo read data
wire                       fifo_empty;  // fifo empty
wire                       fifo_full;   // fifo full
wire [FIFODEPTHLOG2-1:0]   fifo_used;   // fifo data used

// internal logic
reg  [DATAWIDTH-1:0]       address;              // address of sdram
wire [DATAWIDTH-1:0]       end_address;          // end address of sdram
wire                       is_address_sload;     // is sload of address?
wire                       is_increment_address; // is increment address of sdram?
reg  [FIFODEPTHLOG2-1:0]   reads_pending;        // pending reads

// master output
assign avm_m1_address            = address;      // avalon-mm master address
assign avm_m1_byteenable         = 4'b1111;      // avalon-mm master byteenable (32 bit)
assign avm_m1_read               = (!fifo_full); // avalon-mm master read

// export output
assign coe_ltm_export_oLTM_CLK    = ltm_nclk;   // ltm clk
assign coe_ltm_export_oFIFO_FULL  = fifo_full;  // for debug use only (ltm fifo full)
assign coe_ltm_export_oFIFO_EMPTY = fifo_empty; // for debug use only (ltm fifo empty)

// fifo input
assign fifo_wrclk                = csi_clockreset_clk;      // fifo write clk
assign fifo_aclr                 = ~csi_clockreset_reset_n; // fifo asynchronous clear
assign fifo_wrreq                = avm_m1_readdatavalid;    // fifo write request
assign fifo_wrdata               = avm_m1_readdata;         // fifo write data
assign fifo_rdclk                = ltm_nclk;                // fifo read clk
assign fifo_rereq                = read;                    // fifo read request

// internal logic
assign end_address               = ADDRESSBASE + LENGTH * BYTEENABLEWIDTH; // end address of sdram
assign is_address_sload          = (address == end_address);               // is sload of address?
assign is_increment_address      = (!fifo_full) && (!avm_m1_waitrequest);  // is increment address of sdram?
// Todo!! assign fifo_full

// address register
always@(posedge csi_clockreset_clk, negedge csi_clockreset_reset_n) begin
  if (!csi_clockreset_reset_n)
    address <= 0;
  else begin
    if (address == 0)              // initailize
      address <= ADDRESSBASE;
    else if (is_address_sload)     // is sload of address?
      address <= ADDRESSBASE;
    else if (is_increment_address) // is increment address of sdram?
      address <= address + BYTEENABLEWIDTH;
  end
end

// reads_pending register
always@(posedge csi_clockreset_clk, negedge csi_clockreset_reset_n) begin
  if (!csi_clockreset_reset_n)
    reads_pending <= 0;
  else begin
    if (is_increment_address) begin // is increment address of sdram?
      if (!avm_m1_readdatavalid)
        reads_pending <= reads_pending + 1'b1;
    end
    else begin
      if (avm_m1_readdatavalid) // read data valid
        reads_pending <= reads_pending - 1'b1;
    end
  end
end

// ltm clk
ltm_pll ltm_pll0 (
  .inclk0(coe_ltm_export_iCLK_50), // 50MHz
  .c0(ltm_nclk)                    // 33MHz
);

// I2S ltm config
lcd_3wire_config wire0 (
  // Host Side
  .iCLK(coe_ltm_export_iCLK_50),       // 50MHz
  .iRST_n(coe_ltm_export_iRST0),       // reset delay 0
  // 3 wire Side
  .o3WIRE_SCLK(coe_ltm_export_oSCLK),  // I2S clk
  .io3WIRE_SDAT(coe_ltm_export_ioSDAT),// I2S data
  .o3WIRE_SCEN(coe_ltm_export_oSCEN)   // I2s clk enable
);

// tcon controller
touch_tcon tcon0 (
  .iCLK(ltm_nclk),               // 33MHz
  .iRST_n(coe_ltm_export_iRST2), // reset delay 2
  // sdram side
  .iREAD_DATA1({1'b0, fifo_rddata[15:11], fifo_rddata[7:0], 2'b00}),         // G[9:5]B[9:0]
  .iREAD_DATA2({1'b0, fifo_rddata[10:8], 2'b00, fifo_rddata[23:16], 2'b00}), // G[4:0]R[9:0]
  .oREAD_SDRAM_EN(read),         // sdram read request
  // lcd side
  .oLCD_R(coe_ltm_export_oR),    // ltm R
  .oLCD_G(coe_ltm_export_oG),    // ltm G
  .oLCD_B(coe_ltm_export_oB),    // ltm B
  .oHD(coe_ltm_export_oHD),      // ltm h. sync
  .oVD(coe_ltm_export_oVD),      // ltm v.sync
  .oDEN(coe_ltm_export_oDEN)     // ltm data enable
);

// ltm fifo
dcfifo # (
  .lpm_width(DATAWIDTH),     // data width of fifo
  .lpm_numwords(FIFODEPTH),  // fifo depth of fifo
  .lpm_widthu(FIFODEPTHLOG2) // fifo width of depth
) fifo0 (
  .wrclk(fifo_wrclk),        // fifo write clk
  .aclr(fifo_aclr),          // fifo asynchronous clear
  .wrreq(fifo_wrreq),        // fifo write request
  .data(fifo_wrdata),        // fifo write data
  .rdclk(fifo_rdclk),        // fifo read clk
  .rdreq(fifo_rereq),        // fifo read request
  .q(fifo_rddata),           // fifo read data
  .rdempty(fifo_empty),      // fifo empty
  .wrusedw(fifo_used)        // fifo used data
);

endmodule
